/* 
 * Thomas Pantzer: t_vector.cc, v0.1b  8.11.98 0:58 
 * 
 * 
 * Copyright 1998 by Thomas Pantzer
 * 
 * Permission to use, copy, modify, and distribute this software for noncommercial 
 * use and without fee is hereby granted, provided that the above copyright notice 
 * appear in all copies and that both that copyright notice and this permission 
 * notice appear in supporting documentation, and that the name of the Author not be 
 * used in advertising or publicity pertaining to distribution of the software 
 * without specific, written prior permission. The Author makes no representations about
 * the suitability of this software for any purpose.  It is provided "as is" 
 * without expressed or implied warranty. 
 * 
 * 
 * pantec@informatik.uni-leipzig.de   (Thomas Pantzer) 
 * 
 */



#include "t_vector.h"
#include <sys/types.h>

//#ifdef DEBUG
#include <stdio.h>
//#endif

void t_vector2d::Init(class t_vector2d *p)
{
   x = p->x;
   y = p->y;

   X = p->X;
   Y = p->Y;
}

void t_vector2d::Init(int _x, int _y)
{
   x = _x;
   y = _y;

   X = (double)_x;
   Y = (double)_y;
}

void t_vector2d::Init(double _x, double _y)
{
   X = _x;
   Y = _y;

   x = (int)_x;
   y = (int)_y;
}

void t_vector2d::Add(class t_vector2d *p)
{
   X += p->X;
   Y += p->Y;
   
   x = (int)X;
   y = (int)Y;
}

void t_vector2d::Neg(void)
{
   X = -X;
   Y = -Y;
   
   x = -x;
   y = -y;
}



void t_vector2d::Sub(class t_vector2d *p)
{
   X -= p->X;
   Y -= p->Y;

   x = (int)X;
   y = (int)Y;
}

void t_vector2d::Normalize(void)
{
   double d = Length();
   
   X /= d;
   Y /= d;

   x = (int)X;
   y = (int)Y;
}

double t_vector2d::Scalar(class t_vector2d *p)
{
  if (p == NULL)                                                                                
    return 0.0;                                                                                 
                                                                                                
  return ((X*p->X) + (Y*p->Y));
}

double t_vector2d::Length()
{
  return sqrt(fabs(Scalar(this)));
}

void t_vector2d::Mult(double d)
{
   X = X*d;
   Y = Y*d;

   x = (int)X;
   y = (int)Y;
}


double t_vector2d::Distance(class t_vector2d *p)
{
  double d = sqrt( SQR(X - p->X) + SQR(Y - p->Y) );
  
#ifdef DEBUG
  fprintf(stderr,"dist: %g\t(%g,%g)\t(%d,%d)\n",d,p->X,p->Y,p->x,p->y);
#endif  
  return d;
}

int t_vector2d::GetX(void)
{
   return x;
}

int t_vector2d::GetY(void)
{
   return y;
}

void t_vector2d::Turn(t_vector2d *V)
{
/*
  t_vector2d M;
  int k;
  double
      
  M.Init(this);
  M.Sub(V);
  
  k = M.x;
  M.x = -M.y;
  M.y = k;
  
  M.Add(V);
  this->Init(&M);
*/
} 


char t_vector2d::Equal(t_vector2d *V)
{
  if ((x == V->x) && (y == V->y))
     return true;
     
  return false;   
}
 

void t_vector2d::Rotate(t_vector2d *V, t_vector2d &CSR)
{
  t_vector2d M,N;
//  int k;
      
  M.Init(this);
  M.Sub(V);
  
  N.X = (M.X*CSR.X) + (M.Y*CSR.Y);
  N.Y = -(M.X*CSR.Y) + (M.Y*CSR.X);     

  N.Add(V);
  this->Init(&N);
}


double t_vector2d::CosPhi(t_vector2d *v2)                                                            
{                                                                                               
  double c,d,r;                                                                                 
#ifndef FAST                                                                                    
  t_vector2d A,B;                                                                                 
#endif                                                                                          
                                                                                                
  if (v2 == NULL)                                                                               
     return 0.0;                                                                                
                                                                                                
#ifndef FAST                                                                                    
  A.Init(this);                                                                                 
  B.Init(v2);                                                                                   
  A.Normalize();                                                                                
  B.Normalize();                                                                                
#endif                                                                                          
                                                                                                
#ifndef FAST                                                                                    
  d = A.Length() * B.Length();                                                                  
#else                                                                                           
  d = Length() * v2->Length();                                                                  
#endif                                                                                          
                                                                                                
  if (d == 0.0)                                                                                 
     return 0.0;                                                                                
                                                                                                
#ifndef FAST                                                                                    
  if (d != 1.0)
     fprintf(stderr,"len1*Len2 != 1.0 LL:%g\n",d);
     
  c = (A.Scalar(&B) / d);                                                                       
#else                                                                                           
  c = (Scalar(v2) / d );    // == cos(phi)                                                      
#endif                                                                                          

  return c;
}


double t_vector2d::Cross(t_vector2d *P)
{
   return (X * P->Y) - (Y * P->X);
}

void t_vector2d::Print(void)
{
   fprintf(stderr,"(%g,%g)\t(%d,%d)\n",X,Y,x,y);
}
