#ifndef _M105_H
#define _M105_H

/*
 *  Copyright (c) 1994 Thomas Faehnle  (Thomas.Faehnle@student.uni-ulm.de)
 *
 *  m105.h,v 1.7 1994/04/13 22:28:42 blurf Exp
 */

#include "scanner.h"

#define M105_SCANNER_NAME	"Scanner"
#define M105_SCANNER_MAJOR	26	/* our major number */


#include <linux/version.h>

#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,2,0))

#define GI1904_IOBASE		0x2ab	
/* base address of scanner interface       (0x22b, 0x26b, 0x2ab, 0x2eb,
					    0x32b, 0x36b, 0x3ab or 0x3eb) */
#define GI1904_IRQ		10	
/* used IRQ  (3, 5 or 10) */
#define GI1904_DMA		3	
/* used DMA channel  (1 or 3) */

#if GI1904_IRQ == 3
#  define GI1904_IRQ_MASK	0x10
#elif GI1904_IRQ == 5
#  define GI1904_IRQ_MASK	0x20
#elif GI1904_IRQ == 10
#  define GI1904_IRQ_MASK	0x08
#else
#error "I can't use an IRQ line other than 3, 5 or 10!"
#endif

#if GI1904_DMA == 1
#  define GI1904_DMA_MASK	0x01
#elif GI1904_DMA == 3
#  define GI1904_DMA_MASK	0x04
#else
#error "DMA lines other than 1 or 3 can't be used!"
#endif


#endif

/* #define DEFAULT_BUFLINES	13      / * default # of scanlines in buffer */

/****************************************************************************
 *
 * It may be wise not to touch anything below this line...
 *
 ****************************************************************************/

#define MAX_BUFSIZ		(1<<19) /* max. buffer size (bytes), prevents one
					   from locking up all mem as scan buffer 8-) */


/*#define MAX_BLK			(4072 - HEADERSIZ) *//* <--- don't touch this */
					/* max. chunk of mem we can get with kmalloc()
					   note: this is likely to change in the future */



#ifdef WANT_JUNK

  DMA_IRQ       [0|0|0|0|0|0|0|0]
                 | | |     |   |
            IRQ  3 5 10    |   |
                           |   |
                       DMA 3   1
                     
  CTRL          [0|0|0|0|0|1|0|1]
                     |     | | |
                     |   STBY| ON
                     |       2Bit
                     4Bit  
                 
                 
  CG-8000 registers
                 
  MISC0         [1|0|0|0|0|0|1|1]
                 |           | |
                 |           | Color/Mono  
                 |           |
                OK to read   SG/Txt                  

  MISC1 == 0xa5		


  		 1bit=1	 Color=1
                 8bit=0	 Mono=1
		 |	 |
  MISC2         [1|1|0|0|1|1|0|0]
                   |       |
                   |       scan in process/lamp on
		   |                                        
		   100/200=1
		   300/400=0
#endif


#if DEFAULT_BUFLINES >= 800
#error "Well, I think a buffer with >= 500 lines is a bit excessive, isn't it?"
#endif

#define GI1904_DMA_IRQ		GI1904_IOBASE
#define GI1904_CONTROL		(GI1904_IOBASE + 1)

#define GI1904_MISC0		(GI1904_IOBASE + 2)

#define GI1904_MISC1		(GI1904_IOBASE + 3)
#define GI1904_IDENT		(GI1904_IOBASE + 3)

#define GI1904_MISC2		(GI1904_IOBASE + 4)

#define STANDBY			0x05    /*0x05*/
/* / #define OPERATE			0xe1    / * (0x33 | (gi1904_regs.ctl_mask & ~0x05)) */
#define OFF			0

/* /#define OPERATE		0x8 & ~0x04

/ *
 STANDBY 0x05
 OPERATE 0x91

 STANDBY 0x07
 OPERATE 0xe3
 */
 
 
#define GI1904_MAGIC		0xa5
#define GI1904_MODE_MASK	0x7f    /* 0x3f */

/*
static unsigned char m105_modebits[] = { 
                                          0x02, 0x04, 0x07, 0x09
                                         ,0x0c, 0x0e, 0x10, 0x13
                                         ,0x00, 0x00, 0x00, 0x00, 0x00, 0x00 
                                       };
*/
/*
#define m105_modeinfo_struct modeinfo_struct
*/
/*
static struct m105_modeinfo_struct m105_modeinfo[] = {

  ,{ 500, 500, 1,1, 258, 0,0,0,0, 0,0,0,0, 0  }
  ,{ 600, 600, 1,1, 310, 0,0,0,0, 0,0,0,0, 0  }
  ,{ 700, 700, 1,1, 362, 0,0,0,0, 0,0,0,0, 0  }
  ,{ 800, 800, 1,1, 414, 0,0,0,0, 0,0,0,0, 0  }
*/
/* worthless modi */  
  /*  ,{ 100, 100, 8, 3, 36, 0,0,0,0,0,0,0,0,0  }     100 g C      RBG */
  /*  ,{ 200, 200, 8, 3, 72, 0,0,0,0,0,0,0,0,0  }     200 g C      RBG */ 
  /*  ,{ 300, 300, 8, 3, 96, 0,0,0,0,0,0,0,0,0  }     300 Sg C */
  /*  ,{ 400, 400, 8, 3, 96, 0,0,0,0,0,0,0,0,0  }     400 g C */

/* color modi */
/*  ,{ 300, 300, 8, 3,155, 0,0,0,0, 0,0,0,0, SCC_HARDSWITCH }  */
  
/* add modes here here ...*/  

#define NUM_SCANMODES 12

struct t_Scanner {
 int id, num_modes, op_cmd, modeindex;
 
 struct scanner_type         type;
 struct modeinfo_struct      mode[NUM_SCANMODES];
};



/* all supported scanners and their capabilities */


#define CG8400_MASK		0x80
#define CG8400_HALFTONE		0x10
#define CG8400_4BIT		0x10
#define CG8400_1BIT		0x40


static struct t_Scanner SScanner[] = 
     {
      {
       1,4,0x91,0,{"Vobis","Personal 64"},{  
             { 100,100,1,1,  52, 0,0,0,0, 0,0,0,0, SCC_SOFTDETECT, 0x02 }
            ,{ 200,200,1,1, 104, 0,0,0,0, 0,0,0,0, SCC_SOFTDETECT, 0x04 }
            ,{ 300,300,1,1, 156, 0,0,0,0, 0,0,0,0, SCC_SOFTDETECT, 0x07 }
            ,{ 400,400,1,1, 208, 0,0,0,0, 0,0,0,0, SCC_SOFTDETECT, 0x09 }
            ,{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0}
            ,{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0}
            ,{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0}
            ,{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0}
/*            ,{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0}
            ,{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0} 
            ,{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0}
            ,{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0}
*/            ,{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0}
            ,{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0}
            ,{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0}
            ,{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0}
            }
      }      
      
/*
        	       Mono                    Color      
	       txt   ht    g     sg     txt   ht    g     sg
	 100  0xec  0xfc  0xbc  0xac   0xe8  0xf8  0xb8  0xa8
	 200  0xe4  0xf4  0xb4  0xa4   0xe0  0xf0  0xb0  0xa0	
	 300  0xc4  0xd4  0x94  0x84   0xc0  0xd0  0x90  0x80
	 400  0xcc  0xdc  0x9c  0x8c   0xc8  0xd8  0x90  0x88
*/

     ,{2,12,0xe3,0,{"Mustek","CG-8000/8400"},{
            /* bitimage */
             { 100,100,1,1,  52, 0,0,0,0, 0,0,0,0, SCC_SOFTDETECT, 0xec}
            ,{ 200,200,1,1, 104, 0,0,0,0, 0,0,0,0, SCC_SOFTDETECT, 0xe4}
            ,{ 300,300,1,1, 156, 0,0,0,0, 0,0,0,0, SCC_SOFTDETECT, 0xc4}
            ,{ 400,400,1,1, 208, 0,0,0,0, 0,0,0,0, SCC_SOFTDETECT, 0xcc}
            /* graylevel modi */
            ,{ 100, 100, 8, 1,  52, 0,0,0,0, 0,0,0,0, SCC_SOFTDETECT, 0xac}
            ,{ 200, 200, 8, 1, 104, 0,0,0,0, 0,0,0,0, SCC_SOFTDETECT, 0xa4}  
            ,{ 300, 300, 8, 1, 156, 0,0,0,0, 0,0,0,0, SCC_SOFTDETECT, 0x84}   
            ,{ 400, 400, 8, 1, 208, 0,0,0,0, 0,0,0,0, SCC_SOFTDETECT, 0x8c}  
            /* true color */
            ,{ 100, 100, 8, 3, 52, 0,0,0,0, 0,0,0,0, SCC_SOFTDETECT, 0xa8}   
            ,{ 200, 200, 8, 3,104, 0,0,0,0, 0,0,0,0, SCC_SOFTDETECT, 0xa0}   
/*
            ,{ 300, 300, 8, 3,156, 0,0,0,0, 0,0,0,0, SCC_HARDSWITCH, 0}   
            ,{ 400, 400, 8, 3,208, 0,0,0,0, 0,0,0,0, SCC_HARDSWITCH, 0}   
*/
            ,{ 100, 100, 4, 3, 52, 0,0,0,0, 0,0,0,0, SCC_SOFTDETECT, 0xb8}   
            ,{ 200, 200, 4, 3,104, 0,0,0,0, 0,0,0,0, SCC_SOFTDETECT, 0xb0}   
/*
            ,{ 300, 300, 4, 3,156, 0,0,0,0, 0,0,0,0, SCC_HARDSWITCH, 0}   
            ,{ 400, 400, 4, 3,208, 0,0,0,0, 0,0,0,0, SCC_HARDSWITCH, 0}   
*/            
            }
      }
     ,{0,1,0,0,{"unknown","unknown"},{
               {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0}
              ,{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0} 
              ,{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0}
              ,{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0} 
/*              ,{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0}
              ,{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0}
              ,{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0}
              ,{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0}
*/              
              ,{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0}
              ,{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0}
              ,{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0}
              ,{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0}
              ,{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0}
              ,{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0} 
              ,{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0}
              ,{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0}
              }
      }  
     };
 

struct gi1904_regs_struct {
  unsigned char mr0;
  unsigned char mr1;
  unsigned char mr2;
  unsigned char ctl_mask;
};

static struct gi1904_regs_struct gi1904_regs = { 0xce, 0, 0, 0x90 };

/* / #define HEADERSIZ		4096	/ * size of buffer header (take alignment into account) */
/*
struct buffer_block_struct {
  struct buffer_block_struct *next;
  unsigned int blksiz;
  unsigned int blkoff;
  unsigned int blklines;
  unsigned char *data;   
};

#define HEADERSIZE sizeof(struct buffer_block_struct)
*/

struct device_status_struct {
  char device_open:	1;
  char irq_enabled:	1;
  char process_sleeps:	1;
  char buffer_empty:	1;
};

struct buffer_pointer {
  struct buffer_block_struct *blkptr;
  unsigned int index;
};
#endif /* _M105_H */
