


/* kmalloc DMA bucket sizes */

struct t_dma_block {
  char rc,id;
  long unsigned int size,filled,gotten,fullsize;
/*  char _ready;*/
  struct t_dma_block *next; 
  int buggy;
/*  double efficiency; */
  char *data; 
};
                   
struct t_queue { 
  struct t_dma_block *first;
  struct t_dma_block *wptr;
  struct t_dma_block *rptr;
/*  struct t_dma_block *actual;  */
};	               


struct t_dma_block BufHeader[32];
int BufHeaderCount = 0;

#define NR_BUCKETS 4
int km_bucket[NR_BUCKETS] = {8192,16384,32768,65536};

char dma_msg[2][16] = {"ok\0","wrong\0"};

struct t_queue Queue; 

/* Pure 2^n version of get_order */
static __inline__ int __get_order(unsigned long size)
{
	int order;

	size = (size + PAGE_SIZE -1) >> PAGE_SHIFT;
	order = -1;
	do {
		size >>= 1;
		order++;
	} while (size);

	return order;
}

struct t_dma_block *get_memblock(struct t_dma_block *prev, unsigned long int chunk_size)
{
   struct t_dma_block *buff;
   unsigned long int dma_a,dma_o,dma_s;
   int i;
   
/*   buff = (struct t_dma_block*)kmalloc(sizeof(struct t_dma_block) ,GFP_DMA);
   buff = (struct t_dma_block*)__get_dma_pages(GFP_KERNEL,__get_order(HEADERSIZE));
   if (buff == NULL)
      return NULL;
*/
   if (BufHeaderCount >= 32)
      return NULL;
      
   buff = &BufHeader[BufHeaderCount];
   buff->id = BufHeaderCount;
   
/*   buff->data = (char*)kmalloc(chunk_size , GFP_DMA); */
   buff->data = (char*)__get_dma_pages(GFP_KERNEL,__get_order(chunk_size));
   if (buff->data == NULL)
      {
       PRINTK("m105: kmalloc failed\n");
/*       kfree((char*)buff); 
       free_pages((long)buff,GFP_KERNEL); 
*/       
       return NULL;
      }    

   BufHeaderCount++; 
   buff->buggy = 0;
      
   dma_a = (unsigned long int)buff->data;
   dma_o = dma_a % 65536;
   dma_s = 65536 - dma_o;
   
/*   dma_o += chunk_size; */
   if (dma_s < chunk_size)
      {
       PRINTK("m105: dma buffer not aligned properly, offset 0x%05lx\n",dma_o); 
       buff->buggy = 1; 
       buff->size = dma_s;		/* size of mem usable for dma */
      } 
    else  
       buff->size = chunk_size;		/* size of mem usable for dma */
  
   buff->fullsize = chunk_size;
   buff->filled = 0;
   buff->gotten = 0;
   buff->next = prev;

   for (i=0; i<chunk_size; i += 1024)
     buff->data[i] = 0;			/* access the memory the first time */ 	
   
 
/*   j = mlock(buff->data,chunk_size); */
   printk("m105: got %ld bytes @ 0x%08lx %s aligned DMA-block @ 0x%08lx\n",
                  chunk_size,(long)buff->data,&dma_msg[buff->buggy][0],dma_a);
   
   return buff;
}


int recount = 0;

void close_circular(struct t_dma_block *B,struct t_dma_block *R)
{
   recount++;
/*   printk("m105: start make circular step %d\n",recount); */

   if ((B==NULL) || (R==NULL))
      return;

   if (B->next == NULL)
      B->next = R;
    else
      {
       if (B->next == R)
          return;
          
       close_circular(B->next,R);  
      } 
      
   recount--;   
/*   printk("m105: end make circular step %d\n",recount); */
}


struct t_dma_block *allocate_queue_mem(int req_size)
{
    struct t_dma_block *tmp,*prev;
    int i,L,rest_size;
    
    L = 0;
    rest_size = req_size;
    prev = NULL;
    i = NR_BUCKETS;
    while (i > 0)
      {
       i--;
       
       while ((rest_size < km_bucket[i]) && (i>0))
         i--;
         
       tmp = get_memblock(prev,km_bucket[i]);
    
       while (tmp != NULL)
         {
          prev = tmp;
          
          if (prev->buggy == 0)
             {
              L += prev->size;
              rest_size -= prev->size;
             } 
          
          if (L > req_size)
             break;

          tmp = get_memblock(prev,km_bucket[i]);
         } 
         
       if (L > req_size)
          break;
      } 
      
    return prev;
}


unsigned long int proper_queue_mem(struct t_dma_block *B, struct t_dma_block *first)
{
   if ((B == NULL) || (first == NULL))
      return 0;
      
   if (B->buggy != 0)
      return proper_queue_mem(B->next,first);      

   return B->fullsize + proper_queue_mem(B->next,first);      
}

unsigned long int total_queue_mem(struct t_dma_block *B, struct t_dma_block *first)
{
   if ((B == NULL) || (first == NULL))
      return 0;
      
   return B->size + total_queue_mem(B->next,first);      
}

unsigned long int full_queue_mem(struct t_dma_block *B, struct t_dma_block *first)
{
   if ((B == NULL) || (first == NULL))
      return 0;
      
   return B->fullsize + full_queue_mem(B->next,first);      
}

long int total_queue_lines(struct t_dma_block *B, struct t_dma_block *first, int div)
{
   if (B == NULL)
      return 0;
      
   if ((B->next == NULL) || (first == NULL))
      return 0;
      
   if (B->next == first)
      return (B->size/div);
      
   return (B->size/div) + total_queue_lines(B->next,first,div);      
}


void break_circular(struct t_dma_block *B, struct t_dma_block *F)
{
   struct t_dma_block *b;
   int i;
   
   b = B;
   i = 0;
   while ((b->next != F) && (i < 4096))
     {
      b = b->next;
      i++;
     } 

   if (i > 2048)
      printk("m105: looped thru circular buffer without end, shouldn't happen\n");
      
   PRINTK("m105: found and broke circular here\n");

   /* next is the start anchor */

   b->next = NULL;		/* break up the circular queue to a linked list */
}

void delete_memblock(struct t_dma_block *B)
{
   /*kfree(B->data);*/
   free_pages((long)B->data,GFP_KERNEL);

   PRINTK("m105: removing invalid dma block @ 0x%08lx\n",(long)B->data);

   B->data = NULL;
   B->filled = 0;
   B->gotten = 0;
   B->size = 0;
   B->buggy = 1;
   
   /* kfree(B);
   free_pages((long)B,GFP_KERNEL);
   */
}


void free_queue_mem(struct t_dma_block *Cast,struct t_dma_block *R)
{
    if ((Cast == NULL) || (R == NULL))
       return;
 
    if (Cast->next != NULL)
       {
        PRINTK("m105: recursive call\n");
        free_queue_mem(Cast->next,R);
        Cast->next = NULL;
       } 
       
    /* free yourself */   
    if (Cast->data != NULL)
       {
        PRINTK("m105: free page @ 0x%0lx\n",(long)Cast->data);
        free_page((long)Cast->data);
        PRINTK("m105:     .... done\n");
        Cast->data = NULL;
       }
        
    Cast->size = 0;
    Cast->filled = 0;
    Cast->gotten = 0;
}

static __inline__ int free_in_block(struct t_dma_block *B)
{
    return B->size - B->gotten;
}

static __inline__ int bytes_in_queue(void)
{
	int av;
	struct t_dma_block *Ptr;  
  /* 
  ,bool,i;

  cli(); 
  i = Queue.wptr->id;
*/  
  
/*  bool = (); */
  
/*  sti(); 
  if (i != Queue.wptr->id)
     {
      PRINTK("m105: interrupt within biq\n");
      return bytes_in_queue();	
     } 
*/   
	av = Queue.rptr->filled - Queue.rptr->gotten;
	
	if (av > 0)
	   return av;		
	
	Ptr = Queue.rptr->next;
	av = Ptr->filled - Ptr->gotten;

	if (av > 0)
	   {
            Queue.rptr = Queue.rptr->next;
            PRINTK("m105: read_queue(): advanced to next bucket @ 0x%lx\n",(long)Queue.rptr->data);

	    return av;		    
	   } 
	
	return 0;
		    
/*   
PRINTK("m105: wrapped av %d\n",av);
     {
      if (av > 0)
         {
          
          return av;
         } 
         
      return 0;	
      printk("m105: #1 wrong bytes in queue return, shouldn't happen\n");   
     }
  
      
  if (av < 0)
     {
      printk("m105: #2 wrong bytes in queue return, shouldn't happen\n");
      av = 0;
     }
         
  return av;
 */
}



unsigned long int init_queue(void)
{
   unsigned long int  res;
   struct t_dma_block /* *B,*D, */ *R;
   
   R = allocate_queue_mem(msize * 1024);
   
   
   
   if (R == NULL)
      return 0;
  
/*      
   printk("m105: first delete loop\n");   
   while (R->buggy != 0)
     {
      D = R;
      R = R->next;  
      
      delete_memblock(D);
      
      if (R == NULL)
         break;
     } 

   if (R == NULL)
      {
       printk("m105: allocate dma mem failed\n");
       return 0;
      }  
*/

/*
   printk("m105: second delete loop\n");
   B = R;
   while (B != NULL)
     {
      while (B->next->buggy != 0)
        {
         D = B->next;              // about to delete this
         B->next = B->next->next;
      
         delete_memblock(D);
         
         if (B->next == NULL)
            break;
        }
   
      B = B->next;  
     } 
*/ 
   if (R == NULL)
      return 0;

   Queue.first = R;

   res = total_queue_mem(Queue.first,Queue.first);
   printk("m105: full-, usable- and proper memsize %ld/%ld/%ld\n",
       full_queue_mem(Queue.first,Queue.first),res,
          proper_queue_mem(Queue.first,Queue.first));

   PRINTK("m105: about to close circular \n");
   close_circular(Queue.first,Queue.first);     
   
   Queue.rptr = Queue.first;
   Queue.wptr = Queue.first;
   
   return res;
}

void done_queue(void)
{
   break_circular(Queue.first,Queue.first);
   free_queue_mem(Queue.first,Queue.first);
}


/*
struct t_dma_block *queue_advance(struct t_dma_block *old,char rq)
{
   struct t_dma_block *n;
   
   n = old->next;
   
   if (rq == 0)
      n->filled = 0;
    else  
      n->gotten = 0;

   return n;
}
*/
