

#include "edit_objs.h"
#include "stuff.h"
#include "minidraw.h"

#include "copyright.h"

char dragging = false;
char rotating = false;
t_drawgrp *new_group = NULL;
t_vector2d rp1,rp2;



t_drawobj *GetNearestObj(t_vector2d *M)
{
   t_drawobj *D;

   D = RootObj.Distance(sqrt(SQR(VScr.width)+SQR(VScr.height)),M,NULL);

   if (D == NULL)
      return NULL;
       
   switch (D->kind)
     {
      case _basic:
      case _cursor:
//      case _group:
        return NULL;
        
      default:
        break;  
     }
   
   if (D != ActualObj)
      {
       if (ActualObj != NULL)
           ActualObj->ClearSelectedView();

       if (D != NULL)
           D->SetSelectedView();
      }
      
   return D;
}


t_drawobj *Init_Object(e_drawtype o_type, t_vector2d *M)
{
   t_line *L;
   t_circle *C;
   t_rectangle *R;
   t_resistor *W;
   t_polygon *P;

   rp1.Init(M);

   switch (o_type)
     {
      case _line:
	L = new t_line;
	L->Init(&CM[color_map],M,NULL,M,VScr.LineWidth);
	return (t_drawobj *)L;

      case _circle:  
	C = new t_circle;
	C->Init(&CM[color_map],M,NULL,1.0,VScr.LineWidth);
	if (FillStyle > 0)
           C->filled = true;

	return (t_drawobj *)C;

      case _rectangle:
	R = new t_rectangle;
	R->Init(&CM[color_map],M,NULL,M,VScr.LineWidth);

	if (FillStyle > 0)
           R->filled = true;

	return (t_drawobj *)R;   
	
      case _polygon:
      	P = new t_polygon;
      	P->Init(&CM[color_map],M,NULL,VScr.LineWidth,Convex);
      	P->AddLine(M);

	if (FillStyle > 0)
           P->filled = true;

	return (t_drawobj *)P;   

      case _resistor:
	W = new t_resistor;
	W->Init(&CM[color_map],M,"A",810.0);

	return (t_drawobj *)W; 
	
       default:
         return NULL;
     }
       
   return NULL;
}


void Change_Object(e_drawtype o_type, t_vector2d *M)
{
   t_line *L;
   t_circle *C;
   t_rectangle *R;
   t_resistor *W;
   t_vector2d V;
   t_string *S;

   switch (o_type)
     {
      case _line:
        L = (t_line *)ActualObj;
        L->endpoint.Init(M);
        break;

      case _circle:
        C = (t_circle *)ActualObj;
        C->radius = M->Distance(&C->anchor)*2.0; 
        break;

      case _rectangle:
        R = (t_rectangle *)ActualObj;
        R->EndP(M);
        break;  

      case _resistor:
        W = (t_resistor *)ActualObj;
/*
        V.Init(M);
        V.Sub(&W->anchor);
        W->MoveRel(&V);
*/
        W->Anchor(M);        
        break;  
        
      default:
        break;  
     }
}


void Change_Poly(e_drawtype o_type, t_vector2d *M, int evtype)
{
   t_polygon *P = (t_polygon *)ActualObj;
   
      switch(o_type)
        {
         case _polygon:
            P->ChangeLine(M);

            switch (evtype)
              {
               case MotionNotify:
                 break;
                 
               case ButtonRelease:
                 if (rp1.Equal(M) == false)
                    P->AddLine(M); 
                 break;  
              }   
            break;
         
         default:
            break;   
        }   
}

char Add_Object(int evtype,t_vector2d *M, e_drawtype o_type)
{
   new_group = NULL;   
//   VScr.backplane = false;
   char st[64];
   
   switch (evtype)
     {
      case ButtonPress:
        if (ActualObj != NULL)
           {
            if (ActualObj->kind == _polygon)
               {
                return false;
               }
           }
   
        ActualObj = Init_Object(o_type,M);
        
        if (ActualObj == NULL)
           return false;
           
        RootObj.Insert(ActualObj);
        return false;
        break;
                    
        
      case MotionNotify:
        rp2.Init(M);
        rp2.Sub(&rp1);
        
	sprintf(st,"% 4d , % 4d",rp2.GetX(),rp2.GetY());
	fl_set_input(fd_odraw->coord2_inp,st);

//         fprintf(stderr,"in motion\n");
      case ButtonRelease:
        if (ActualObj == NULL)
           return false;
           
        if (ActualObj->kind != o_type)
           return false;   
          
        ActualObj->Hide();   
        Change_Object(o_type,M);
        Change_Poly(o_type,M,evtype);
        ActualObj->Show();

        if (o_type == _polygon)
           {
            fprintf(stderr,"origin dist: %g\n",rp1.Distance(M)); 
           } 
        
        if (evtype == MotionNotify) 
           return false;
            
        // object is initialized as opaque    
        ActualObj->ClearOpaqueView();
        
        if (o_type == _polygon)
           {
            if (rp1.Distance(M) < 1.0 ) 
               return true;
           } 
           
        return false;
     }
     
   return false;    
}



char AddText(XEvent *ev, t_vector2d *M)
{
   t_string *S;
   t_vector2d V;
   
   new_group = NULL;   
   switch (ev->type)
     {
      case ButtonPress:
#ifdef DEBUG            
        fprintf(stderr,"new text\n");
#endif
        S = new t_string;
        S->Init(&CM[color_map],M,NULL,VScr.TextFont);
        
        ActualObj = (t_drawobj *)S;
        RootObj.Insert(S);
//        RootObj.SetSelected(true,S);
        addingtext = true;
//        VScr.backplane = false;
        return false;
        break;
                      
      case KeyPress:
        {
         KeySym keysym;
//         XEvent report;
         XComposeStatus compose;
         int buffer_size;
         char buffer[20];
         int charcount;

         buffer_size = 20;

         charcount = XLookupString(&ev->xkey, 
                                    buffer, buffer_size, 
                                   &keysym,
                                   &compose);
                                   
          if (keysym != NoSymbol)
             {
              S = (t_string *)ActualObj;
#ifdef DEBUG
              fprintf(stderr,"nosymbol S=0x%08lx\n",(long)S);
#endif
              if (S == NULL)
                 return false;
                 
              if (S->kind != _string)
                 {
                  addingtext = false; 
                  return false;
                 } 
                  
              S->Hide();

              switch (keysym)
                {
      	         case XK_Delete:
                 case XK_BackSpace:
                   S->Trunc();
                   break;
                   
                 case XK_Escape:
                   addingtext = false;
                   RootObj.Delete(S);
                   ActualObj = NULL;
                   return false;
                   break;

      	         case XK_Return:
                 case XK_Linefeed:
//                   RootObj.SetSelected(false); // selected = false; 
                   S->Opaque(false);
                   S->Show();
                   addingtext = false;
                   return true;
                   break;
                   
                 default:
#ifdef DEBUG
                   fprintf(stderr,"default keysym\n");
#endif
                   if ((keysym < 255) && (keysym >= 32)) 
                      {
                       S->Add((char)keysym);
#ifdef DEBUG
                       fprintf(stderr,"0x%02x [%c] %d\n",(char)keysym,(char)keysym,(char)keysym); 
#endif                       
                      } 
                   break;    
                } // switch kysym
                
              S->Show();            
              return false;
             } // if keysym != NoSymbol
                                       
//        return (int) keysym;
        }
        break;  
     } 
     
   return false;    
}



char Delete_Obj(int evtype, t_vector2d *M)
{
   new_group = NULL;   

   ActualObj = GetNearestObj(M);
                
   if (ActualObj == NULL)
      return false;
                 
   if (evtype == ButtonRelease)   
      {
       ActualObj->Hide();
       RootObj.Delete(ActualObj);

       ActualObj = NULL;
       return true;
      }
      
   return false;   
}


char Rotate_Obj(int evtype, t_vector2d *M)
{
   new_group = NULL;   

   switch (evtype)
     {   
      case ButtonPress:
        if ((ActualObj = GetNearestObj(M)) != NULL)
           {
            ActualObj->SetOpaqueView();
            rp1.Init(M);
            
            rotating = true;
           } 
        break;
        
      case MotionNotify:
        if (rotating == true)
           {   // drag the selected object
            if (ActualObj == NULL)
               return false;
            
            t_vector2d *a = ActualObj->Anchor();
            rp1.Sub(a);
            rp1.Normalize();
            
            rp2.Init(M);
            rp2.Sub(a);
            rp2.Normalize();
            
            t_vector2d CS;
            double c = rp2.Scalar(&rp1);
            double s = rp2.Cross(&rp1);
            
#ifdef DEBUG
            fprintf(stderr,"atan(%g,%g) = %g\n",s,c,DEG(atan2(s,c)));
#endif            
            CS.Init(c,s);
            
            ActualObj->Hide();
            ActualObj->Rotate(a,CS);
            ActualObj->Show();
            
            rp1.Init(M);
            
//      	    ActualObj->MoveTo(M);
           }
         else  // switch between objects
            ActualObj = GetNearestObj(M);
        break;
        
      case ButtonRelease:
        if (rotating == true)
           {
            rotating = false;
            ActualObj->ClearOpaqueView();
           } 
 
        return true;   
        break;
     }   

   return false;   
}


char Upper_Obj(int evtype, t_vector2d *M)
{
   new_group = NULL;   

#ifdef DEBUG_EDIT     
   fprintf(stdout,"distance %g, object 0x%lx, kind=%d\n",
          D->anchor.Distance(M),(long)D,D->kind);
#endif

   ActualObj = GetNearestObj(M); 
             
   if (ActualObj == NULL)
      return false;
                 
   if (evtype == ButtonRelease)   
      {
       ActualObj->Hide();
       RootObj.Unchain(ActualObj); // pick it out
       RootObj.Insert(ActualObj);  // insert at last
       ActualObj->Show();
       ActualObj = NULL;
       
       return true;
      }
      
   return false;   
}


char Group_Obj(int evtype, t_vector2d *M)
{
   ActualObj = GetNearestObj(M);

   if (ActualObj == NULL)
      return false;
                 
   if (evtype == ButtonRelease)   
      {
       if (new_group == NULL)
          {
           new_group = new t_drawgrp;
           new_group->Init(&CM[color_map],&ActualObj->anchor, NULL, 1);
           RootObj.Insert(new_group);
           new_group->Opaque(false);
          }
           
       if (new_group != ActualObj)
          {
	   ActualObj->ClearSelectedView();

           RootObj.Unchain(ActualObj); // pick it out
           new_group->Add(ActualObj);
       
           new_group->SetSelectedView();
          } 

       ActualObj = NULL;
       
       // return true
       return false;
      }
      
   return false;   
}


char Move_Obj(int evtype,t_vector2d *M)
{
   new_group = NULL;
   
//   M->Print();

   switch (evtype)
     {
      case ButtonPress:
        if ((ActualObj = GetNearestObj(M)) != NULL)
           {
            dragging = true;
            ActualObj->SetOpaqueView();
//            rp1.Init(M);
           } 
        break;
        
      case MotionNotify:
        if (dragging == true)
           {   // drag the selected object
            if (ActualObj == NULL)
               return false;
            
//            rp1.Neg();
//            rp1.Add(M);
//      	    ActualObj->Hide();
      	    ActualObj->MoveTo(M);
//      	    ActualObj->Show();
           }
         else  // switch between objects
            ActualObj = GetNearestObj(M);
        break;
        
      case ButtonRelease:
        if (dragging == true)
           {
            dragging = false;
            ActualObj->ClearOpaqueView();
           } 
 
        return true;   
        break;
     }      	
      
   return false;  
}

char Save_Obj(int evtype,t_vector2d *M)
{
   new_group = NULL;

   switch (evtype)
     {
      case ButtonPress:
        if ((ActualObj = GetNearestObj(M)) != NULL)
           {
//            dragging = true;
//            ActualObj->SetOpaqueView();
            const char *fname;
            
            if ((fname = fl_show_file_selector("Export selected Object To","","*.dobj","")))
               {
                int fd = open(fname,O_RDWR|O_CREAT|O_TRUNC,0644);
                ActualObj->SaveTo(fd);
                close(fd);
               }
               
           } 
        break;
        
      case MotionNotify:
        ActualObj = GetNearestObj(M);
        break;
        
      case ButtonRelease:
        RootObj.SelectAll(false);
        return true;   
        break;
     }      	
      
   return false;  
}
